@@ -39,22 +39,23 @@
 #include "shrpx_error.h"
 #include "shrpx_http2_downstream_connection.h"
 #include "shrpx_client_handler.h"
-#include "shrpx_ssl.h"
+#include "shrpx_tls.h"
 #include "shrpx_http.h"
 #include "shrpx_worker.h"
 #include "shrpx_connect_blocker.h"
+#include "shrpx_log.h"
 #include "http2.h"
 #include "util.h"
 #include "base64.h"
-#include "ssl.h"
+#include "tls.h"
 
 using namespace nghttp2;
 
 namespace shrpx {
 
 namespace {
-const ev_tstamp CONNCHK_TIMEOUT = 5.;
-const ev_tstamp CONNCHK_PING_TIMEOUT = 1.;
+constexpr ev_tstamp CONNCHK_TIMEOUT = 5.;
+constexpr ev_tstamp CONNCHK_PING_TIMEOUT = 1.;
 } // namespace
 
 namespace {
@@ -95,7 +96,7 @@ void settings_timeout_cb(struct ev_loop *loop, ev_timer *w, int revents) {
     SSLOG(INFO, http2session) << "SETTINGS timeout";
   }
 
-  downstream_failure(http2session->get_addr());
+  downstream_failure(http2session->get_addr(), http2session->get_raddr());
 
   if (http2session->terminate_session(NGHTTP2_SETTINGS_TIMEOUT) != 0) {
     delete http2session;
@@ -137,13 +138,6 @@ void readcb(struct ev_loop *loop, ev_io *w, int revents) {
     return;
   }
   http2session->connection_alive();
-
-  rv = http2session->do_write();
-  if (rv != 0) {
-    delete http2session;
-
-    return;
-  }
 }
 } // namespace
 
@@ -202,6 +196,7 @@ Http2Session::Http2Session(struct ev_loop *loop, SSL_CTX *ssl_ctx,
       group_(group),
       addr_(addr),
       session_(nullptr),
+      raddr_(nullptr),
       state_(DISCONNECTED),
       connection_check_state_(CONNECTION_CHECK_NONE),
       freelist_zone_(FREELIST_ZONE_NONE) {
@@ -244,6 +239,11 @@ int Http2Session::disconnect(bool hard) {
 
   wb_.reset();
 
+  if (dns_query_) {
+    auto dns_tracker = worker_->get_dns_tracker();
+    dns_tracker->cancel(dns_query_.get());
+  }
+
   conn_.rlimit.stopw();
   conn_.wlimit.stopw();
 
@@ -270,8 +270,8 @@ int Http2Session::disconnect(bool hard) {
   // When deleting Http2DownstreamConnection, it calls this object's
   // remove_downstream_connection().  The multiple
   // Http2DownstreamConnection objects belong to the same
-  // ClientHandler object if upstream is h2 or SPDY.  So be careful
-  // when you delete ClientHandler here.
+  // ClientHandler object if upstream is h2.  So be careful when you
+  // delete ClientHandler here.
   //
   // We allow creating new pending Http2DownstreamConnection with this
   // object.  Upstream::on_downstream_reset() may add
@@ -302,12 +302,48 @@ int Http2Session::disconnect(bool hard) {
   return 0;
 }
 
+int Http2Session::resolve_name() {
+  int rv;
+
+  auto dns_query = make_unique<DNSQuery>(
+      addr_->host, [this](int status, const Address *result) {
+        int rv;
+
+        if (status == DNS_STATUS_OK) {
+          *resolved_addr_ = *result;
+          util::set_port(*this->resolved_addr_, this->addr_->port);
+        }
+
+        rv = this->initiate_connection();
+        if (rv != 0) {
+          delete this;
+        }
+      });
+  resolved_addr_ = make_unique<Address>();
+  auto dns_tracker = worker_->get_dns_tracker();
+  rv = dns_tracker->resolve(resolved_addr_.get(), dns_query.get());
+  switch (rv) {
+  case DNS_STATUS_ERROR:
+    return -1;
+  case DNS_STATUS_RUNNING:
+    dns_query_ = std::move(dns_query);
+    state_ = RESOLVING_NAME;
+    return 0;
+  case DNS_STATUS_OK:
+    util::set_port(*resolved_addr_, addr_->port);
+    return 0;
+  default:
+    assert(0);
+    abort();
+  }
+}
+
 int Http2Session::initiate_connection() {
   int rv = 0;
 
   auto worker_blocker = worker_->get_connect_blocker();
 
-  if (state_ == DISCONNECTED) {
+  if (state_ == DISCONNECTED || state_ == RESOLVING_NAME) {
     if (worker_blocker->blocked()) {
       if (LOG_ENABLED(INFO)) {
         SSLOG(INFO, this)
@@ -350,6 +386,8 @@ int Http2Session::initiate_connection() {
       return -1;
     }
 
+    raddr_ = &proxy.addr;
+
     worker_blocker->on_success();
 
     ev_io_set(&conn_.rev, conn_.fd, EV_READ);
@@ -374,36 +412,69 @@ int Http2Session::initiate_connection() {
     return 0;
   }
 
-  if (state_ == DISCONNECTED || state_ == PROXY_CONNECTED) {
+  if (state_ == DISCONNECTED || state_ == PROXY_CONNECTED ||
+      state_ == RESOLVING_NAME) {
     if (LOG_ENABLED(INFO)) {
-      SSLOG(INFO, this) << "Connecting to downstream server";
+      if (state_ != RESOLVING_NAME) {
+        SSLOG(INFO, this) << "Connecting to downstream server";
+      }
     }
     if (addr_->tls) {
       assert(ssl_ctx_);
 
-      auto ssl = ssl::create_ssl(ssl_ctx_);
-      if (!ssl) {
-        return -1;
+      if (state_ != RESOLVING_NAME) {
+        auto ssl = tls::create_ssl(ssl_ctx_);
+        if (!ssl) {
+          return -1;
+        }
+
+        tls::setup_downstream_http2_alpn(ssl);
+
+        conn_.set_ssl(ssl);
+        conn_.tls.client_session_cache = &addr_->tls_session_cache;
+
+        auto sni_name =
+            addr_->sni.empty() ? StringRef{addr_->host} : StringRef{addr_->sni};
+
+        if (!util::numeric_host(sni_name.c_str())) {
+          // TLS extensions: SNI. There is no documentation about the return
+          // code for this function (actually this is macro wrapping SSL_ctrl
+          // at the time of this writing).
+          SSL_set_tlsext_host_name(conn_.tls.ssl, sni_name.c_str());
+        }
+
+        auto tls_session = tls::reuse_tls_session(addr_->tls_session_cache);
+        if (tls_session) {
+          SSL_set_session(conn_.tls.ssl, tls_session);
+          SSL_SESSION_free(tls_session);
+        }
       }
 
-      ssl::setup_downstream_http2_alpn(ssl);
-
-      conn_.set_ssl(ssl);
-
-      auto sni_name =
-          addr_->sni.empty() ? StringRef{addr_->host} : StringRef{addr_->sni};
-
-      if (!util::numeric_host(sni_name.c_str())) {
-        // TLS extensions: SNI. There is no documentation about the return
-        // code for this function (actually this is macro wrapping SSL_ctrl
-        // at the time of this writing).
-        SSL_set_tlsext_host_name(conn_.tls.ssl, sni_name.c_str());
+      if (state_ == DISCONNECTED) {
+        if (addr_->dns) {
+          rv = resolve_name();
+          if (rv != 0) {
+            downstream_failure(addr_, nullptr);
+            return -1;
+          }
+          if (state_ == RESOLVING_NAME) {
+            return 0;
+          }
+          raddr_ = resolved_addr_.get();
+        } else {
+          raddr_ = &addr_->addr;
+        }
       }
 
-      auto tls_session = ssl::reuse_tls_session(addr_->tls_session_cache);
-      if (tls_session) {
-        SSL_set_session(conn_.tls.ssl, tls_session);
-        SSL_SESSION_free(tls_session);
+      if (state_ == RESOLVING_NAME) {
+        if (dns_query_->status == DNS_STATUS_ERROR) {
+          downstream_failure(addr_, nullptr);
+          return -1;
+        }
+        assert(dns_query_->status == DNS_STATUS_OK);
+        state_ = DISCONNECTED;
+        dns_query_.reset();
+        raddr_ = resolved_addr_.get();
       }
 
       // If state_ == PROXY_CONNECTED, we has connected to the proxy
@@ -411,12 +482,11 @@ int Http2Session::initiate_connection() {
       if (state_ == DISCONNECTED) {
         assert(conn_.fd == -1);
 
-        conn_.fd =
-            util::create_nonblock_socket(addr_->addr.su.storage.ss_family);
+        conn_.fd = util::create_nonblock_socket(raddr_->su.storage.ss_family);
         if (conn_.fd == -1) {
           auto error = errno;
           SSLOG(WARN, this)
-              << "socket() failed; addr=" << util::to_numeric_addr(&addr_->addr)
+              << "socket() failed; addr=" << util::to_numeric_addr(raddr_)
               << ", errno=" << error;
 
           worker_blocker->on_failure();
@@ -427,15 +497,14 @@ int Http2Session::initiate_connection() {
 
         rv = connect(conn_.fd,
                      // TODO maybe not thread-safe?
-                     const_cast<sockaddr *>(&addr_->addr.su.sa),
-                     addr_->addr.len);
+                     const_cast<sockaddr *>(&raddr_->su.sa), raddr_->len);
         if (rv != 0 && errno != EINPROGRESS) {
           auto error = errno;
-          SSLOG(WARN, this) << "connect() failed; addr="
-                            << util::to_numeric_addr(&addr_->addr)
-                            << ", errno=" << error;
+          SSLOG(WARN, this)
+              << "connect() failed; addr=" << util::to_numeric_addr(raddr_)
+              << ", errno=" << error;
 
-          downstream_failure(addr_);
+          downstream_failure(addr_, raddr_);
           return -1;
         }
 
@@ -445,17 +514,44 @@ int Http2Session::initiate_connection() {
 
       conn_.prepare_client_handshake();
     } else {
+      if (state_ == DISCONNECTED) {
+        // Without TLS and proxy.
+        if (addr_->dns) {
+          rv = resolve_name();
+          if (rv != 0) {
+            downstream_failure(addr_, nullptr);
+            return -1;
+          }
+          if (state_ == RESOLVING_NAME) {
+            return 0;
+          }
+          raddr_ = resolved_addr_.get();
+        } else {
+          raddr_ = &addr_->addr;
+        }
+      }
+
+      if (state_ == RESOLVING_NAME) {
+        if (dns_query_->status == DNS_STATUS_ERROR) {
+          downstream_failure(addr_, nullptr);
+          return -1;
+        }
+        assert(dns_query_->status == DNS_STATUS_OK);
+        state_ = DISCONNECTED;
+        dns_query_.reset();
+        raddr_ = resolved_addr_.get();
+      }
+
       if (state_ == DISCONNECTED) {
         // Without TLS and proxy.
         assert(conn_.fd == -1);
 
-        conn_.fd =
-            util::create_nonblock_socket(addr_->addr.su.storage.ss_family);
+        conn_.fd = util::create_nonblock_socket(raddr_->su.storage.ss_family);
 
         if (conn_.fd == -1) {
           auto error = errno;
           SSLOG(WARN, this)
-              << "socket() failed; addr=" << util::to_numeric_addr(&addr_->addr)
+              << "socket() failed; addr=" << util::to_numeric_addr(raddr_)
               << ", errno=" << error;
 
           worker_blocker->on_failure();
@@ -464,15 +560,15 @@ int Http2Session::initiate_connection() {
 
         worker_blocker->on_success();
 
-        rv = connect(conn_.fd, const_cast<sockaddr *>(&addr_->addr.su.sa),
-                     addr_->addr.len);
+        rv = connect(conn_.fd, const_cast<sockaddr *>(&raddr_->su.sa),
+                     raddr_->len);
         if (rv != 0 && errno != EINPROGRESS) {
           auto error = errno;
-          SSLOG(WARN, this) << "connect() failed; addr="
-                            << util::to_numeric_addr(&addr_->addr)
-                            << ", errno=" << error;
+          SSLOG(WARN, this)
+              << "connect() failed; addr=" << util::to_numeric_addr(raddr_)
+              << ", errno=" << error;
 
-          downstream_failure(addr_);
+          downstream_failure(addr_, raddr_);
           return -1;
         }
 
@@ -481,11 +577,11 @@ int Http2Session::initiate_connection() {
       }
     }
 
-    on_write_ = &Http2Session::downstream_write;
-    on_read_ = &Http2Session::downstream_read;
-
     // We have been already connected when no TLS and proxy is used.
     if (state_ == PROXY_CONNECTED) {
+      on_read_ = &Http2Session::read_noop;
+      on_write_ = &Http2Session::write_noop;
+
       return connected();
     }
 
@@ -501,7 +597,8 @@ int Http2Session::initiate_connection() {
   }
 
   // Unreachable
-  DIE();
+  assert(0);
+
   return 0;
 }
 
@@ -538,7 +635,7 @@ int htp_hdrs_completecb(http_parser *htp) {
 } // namespace
 
 namespace {
-http_parser_settings htp_hooks = {
+constexpr http_parser_settings htp_hooks = {
     nullptr,             // http_cb      on_message_begin;
     nullptr,             // http_data_cb on_url;
     nullptr,             // http_data_cb on_status;
@@ -720,9 +817,9 @@ int on_stream_close_callback(nghttp2_session *session, int32_t stream_id,
                              uint32_t error_code, void *user_data) {
   auto http2session = static_cast<Http2Session *>(user_data);
   if (LOG_ENABLED(INFO)) {
-    SSLOG(INFO, http2session) << "Stream stream_id=" << stream_id
-                              << " is being closed with error code "
-                              << error_code;
+    SSLOG(INFO, http2session)
+        << "Stream stream_id=" << stream_id
+        << " is being closed with error code " << error_code;
   }
   auto sd = static_cast<StreamData *>(
       nghttp2_session_get_stream_user_data(session, stream_id));
@@ -990,11 +1087,15 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
   int rv;
 
   auto upstream = downstream->get_upstream();
+  auto handler = upstream->get_client_handler();
   const auto &req = downstream->request();
   auto &resp = downstream->response();
 
   auto &nva = resp.fs.headers();
 
+  auto config = get_config();
+  auto &loggingconf = config->logging;
+
   downstream->set_expect_final_response(false);
 
   auto status = resp.fs.header(http2::HD__STATUS);
@@ -1044,16 +1145,16 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
 
   if (downstream->get_upgraded()) {
     resp.connection_close = true;
-    // On upgrade sucess, both ends can send data
+    // On upgrade success, both ends can send data
     if (upstream->resume_read(SHRPX_NO_BUFFER, downstream, 0) != 0) {
       // If resume_read fails, just drop connection. Not ideal.
-      delete upstream->get_client_handler();
+      delete handler;
       return -1;
     }
     downstream->set_request_state(Downstream::HEADER_COMPLETE);
     if (LOG_ENABLED(INFO)) {
-      SSLOG(INFO, http2session) << "HTTP upgrade success. stream_id="
-                                << frame->hd.stream_id;
+      SSLOG(INFO, http2session)
+          << "HTTP upgrade success. stream_id=" << frame->hd.stream_id;
     }
   } else {
     auto content_length = resp.fs.header(http2::HD_CONTENT_LENGTH);
@@ -1084,6 +1185,11 @@ int on_response_headers(Http2Session *http2session, Downstream *downstream,
     resp.headers_only = true;
   }
 
+  if (loggingconf.access.write_early && downstream->accesslog_ready()) {
+    handler->write_accesslog(downstream);
+    downstream->set_accesslog_written(true);
+  }
+
   rv = upstream->on_downstream_header_complete(downstream);
   if (rv != 0) {
     // Handling early return (in other words, response was hijacked by
@@ -1414,7 +1520,7 @@ int on_frame_not_send_callback(nghttp2_session *session,
 
     if (upstream->on_downstream_reset(downstream, false)) {
       // This should be done for h1 upstream only.  Deleting
-      // ClientHandler for h2 or SPDY upstream may lead to crash.
+      // ClientHandler for h2 upstream may lead to crash.
       delete upstream->get_client_handler();
     }
 
@@ -1462,7 +1568,11 @@ int send_data_callback(nghttp2_session *session, nghttp2_frame *frame,
 
   wb->append(PADDING.data(), padlen);
 
-  downstream->reset_downstream_wtimer();
+  if (input->rleft() == 0) {
+    downstream->disable_downstream_wtimer();
+  } else {
+    downstream->reset_downstream_wtimer();
+  }
 
   if (length > 0) {
     // This is important because it will handle flow control
@@ -1532,11 +1642,16 @@ int Http2Session::connection_made() {
 
   state_ = Http2Session::CONNECTED;
 
+  on_write_ = &Http2Session::downstream_write;
+  on_read_ = &Http2Session::downstream_read;
+
   if (addr_->tls) {
     const unsigned char *next_proto = nullptr;
     unsigned int next_proto_len = 0;
 
+#ifndef OPENSSL_NO_NEXTPROTONEG
     SSL_get0_next_proto_negotiated(conn_.tls.ssl, &next_proto, &next_proto_len);
+#endif // !OPENSSL_NO_NEXTPROTONEG
 #if OPENSSL_VERSION_NUMBER >= 0x10002000L
     if (!next_proto) {
       SSL_get0_alpn_selected(conn_.tls.ssl, &next_proto, &next_proto_len);
@@ -1544,7 +1659,7 @@ int Http2Session::connection_made() {
 #endif // OPENSSL_VERSION_NUMBER >= 0x10002000L
 
     if (!next_proto) {
-      downstream_failure(addr_);
+      downstream_failure(addr_, raddr_);
       return -1;
     }
 
@@ -1553,7 +1668,7 @@ int Http2Session::connection_made() {
       SSLOG(INFO, this) << "Negotiated next protocol: " << proto;
     }
     if (!util::check_h2_is_selected(proto)) {
-      downstream_failure(addr_);
+      downstream_failure(addr_, raddr_);
       return -1;
     }
   }
@@ -1602,24 +1717,9 @@ int Http2Session::connection_made() {
     return -1;
   }
 
-  auto must_terminate =
-      addr_->tls && !nghttp2::ssl::check_http2_requirement(conn_.tls.ssl);
-
   reset_connection_check_timer(CONNCHK_TIMEOUT);
 
-  if (must_terminate) {
-    if (LOG_ENABLED(INFO)) {
-      LOG(INFO) << "TLSv1.2 was not negotiated. HTTP/2 must not be negotiated.";
-    }
-
-    rv = terminate_session(NGHTTP2_INADEQUATE_SECURITY);
-
-    if (rv != 0) {
-      return -1;
-    }
-  } else {
-    submit_pending_requests();
-  }
+  submit_pending_requests();
 
   signal_write();
   return 0;
@@ -1842,10 +1942,10 @@ int Http2Session::connected() {
   auto sock_error = util::get_socket_error(conn_.fd);
   if (sock_error != 0) {
     SSLOG(WARN, this) << "Backend connect failed; addr="
-                      << util::to_numeric_addr(&addr_->addr)
+                      << util::to_numeric_addr(raddr_)
                       << ": errno=" << sock_error;
 
-    downstream_failure(addr_);
+    downstream_failure(addr_, raddr_);
 
     return -1;
   }
@@ -1894,7 +1994,7 @@ int Http2Session::read_clear() {
     auto nread = conn_.read_clear(buf.data(), buf.size());
 
     if (nread == 0) {
-      return 0;
+      return write_clear();
     }
 
     if (nread < 0) {
@@ -1922,7 +2022,12 @@ int Http2Session::write_clear() {
       }
 
       if (nwrite < 0) {
-        return nwrite;
+        // We may have pending data in receive buffer which may
+        // contain part of response body.  So keep reading.  Invoke
+        // read event to get read(2) error just in case.
+        ev_feed_event(conn_.loop, &conn_.rev, EV_READ);
+        write_ = &Http2Session::write_void;
+        break;
       }
 
       wb_.drain(nwrite);
@@ -1955,7 +2060,7 @@ int Http2Session::tls_handshake() {
   }
 
   if (rv < 0) {
-    downstream_failure(addr_);
+    downstream_failure(addr_, raddr_);
 
     return rv;
   }
@@ -1965,20 +2070,12 @@ int Http2Session::tls_handshake() {
   }
 
   if (!get_config()->tls.insecure &&
-      ssl::check_cert(conn_.tls.ssl, addr_) != 0) {
-    downstream_failure(addr_);
+      tls::check_cert(conn_.tls.ssl, addr_, raddr_) != 0) {
+    downstream_failure(addr_, raddr_);
 
     return -1;
   }
 
-  if (!SSL_session_reused(conn_.tls.ssl)) {
-    auto tls_session = SSL_get0_session(conn_.tls.ssl);
-    if (tls_session) {
-      ssl::try_cache_tls_session(addr_->tls_session_cache, addr_->addr,
-                                 tls_session, ev_now(conn_.loop));
-    }
-  }
-
   read_ = &Http2Session::read_tls;
   write_ = &Http2Session::write_tls;
 
@@ -2001,7 +2098,7 @@ int Http2Session::read_tls() {
     auto nread = conn_.read_tls(buf.data(), buf.size());
 
     if (nread == 0) {
-      return 0;
+      return write_tls();
     }
 
     if (nread < 0) {
@@ -2024,7 +2121,10 @@ int Http2Session::write_tls() {
   for (;;) {
     if (wb_.rleft() > 0) {
       auto iovcnt = wb_.riovec(&iov, 1);
-      assert(iovcnt == 1);
+      if (iovcnt != 1) {
+        assert(0);
+        return -1;
+      }
       auto nwrite = conn_.write_tls(iov.iov_base, iov.iov_len);
 
       if (nwrite == 0) {
@@ -2032,7 +2132,12 @@ int Http2Session::write_tls() {
       }
 
       if (nwrite < 0) {
-        return nwrite;
+        // We may have pending data in receive buffer which may
+        // contain part of response body.  So keep reading.  Invoke
+        // read event to get read(2) error just in case.
+        ev_feed_event(conn_.loop, &conn_.rev, EV_READ);
+        write_ = &Http2Session::write_void;
+        break;
       }
 
       wb_.drain(nwrite);
@@ -2055,6 +2160,11 @@ int Http2Session::write_tls() {
   return 0;
 }
 
+int Http2Session::write_void() {
+  conn_.wlimit.stopw();
+  return 0;
+}
+
 bool Http2Session::should_hard_fail() const {
   switch (state_) {
   case PROXY_CONNECTING:
@@ -2262,9 +2372,9 @@ void Http2Session::on_timeout() {
   }
   case CONNECTING: {
     SSLOG(WARN, this) << "Connect time out; addr="
-                      << util::to_numeric_addr(&addr_->addr);
+                      << util::to_numeric_addr(raddr_);
 
-    downstream_failure(addr_);
+    downstream_failure(addr_, raddr_);
     break;
   }
   }
@@ -2284,4 +2394,6 @@ void Http2Session::check_retire() {
   signal_write();
 }
 
+const Address *Http2Session::get_raddr() const { return raddr_; }
+
 } // namespace shrpx
